//
//  (C)2013 Mobiq, Inc.
//

#import "Global.h"
#import "AppDelegate.h"
#import "SApplicationMenu.h"
#import "SDeployMonitor.h"

static NSUInteger const kTagRecentAppsMenu = 100;

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    recentApplications = [NSMutableArray array];
    _deploys = [NSMutableArray array];
}

-(void)awakeFromNib {

    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    token = [defaults objectForKey:@"token"];
    if (token == nil) {
        token = @"";
    }

    _statusMenu = [[NSMenu alloc] init];
    _statusMenu.delegate = self;

    _statusItem = [[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength];
    [_statusItem setImage:[NSImage imageNamed:@"upload-cloud"]];

    [_statusItem setHighlightMode:YES];
    [_statusItem setMenu:_statusMenu];

    NSMenuItem *menuItem = nil;

    menuItem = [_statusMenu addItemWithTitle:@"No recent items" action:nil keyEquivalent:@""];
    [menuItem setImage:[NSImage imageNamed:@"clock"]];
    menuItem.tag = 10;

    [_statusMenu addItem:[NSMenuItem separatorItem]];
    menuItem = [_statusMenu addItemWithTitle:NSLocalizedString(@"All Applications", nil) action:@selector(preferencesAction:) keyEquivalent:@""];
    [menuItem setImage:[NSImage imageNamed:@"block"]];
    [menuItem setTag:kTagRecentAppsMenu];
    [_statusMenu addItem:[NSMenuItem separatorItem]];
    menuItem = [_statusMenu addItemWithTitle:NSLocalizedString(@"Preferences...", nil) action:@selector(preferencesAction:) keyEquivalent:@""];
    [menuItem setImage:[NSImage imageNamed:@"gear"]];
    [_statusMenu addItemWithTitle:NSLocalizedString(@"Quit", nil) action:@selector(quitAction:) keyEquivalent:@""];

    [[NSUserNotificationCenter defaultUserNotificationCenter] setDelegate:self];
    
    if ([token isEqualToString:@""]) {
        [self preferencesAction:nil];
    }
}

- (BOOL)userNotificationCenter:(NSUserNotificationCenter *)center shouldPresentNotification:(NSUserNotification *)notification {
    return YES;
}

- (void)menuWillOpen:(NSMenu *)menu {

    if ([menu isEqual:_statusMenu]) {

        [[[SRequest alloc] initWithToken:token apiPath:@"applications" delegate:self] start];

        while (true) {
            NSMenuItem *item = [_statusMenu itemWithTag:55];
            if (!item) break;
            [_statusMenu removeItem:item];
        }

        if ([recentApplications count] != 0 && [_statusMenu itemWithTag:10] != nil) {
            [_statusMenu removeItem:[_statusMenu itemWithTag:10]];
        }

        for (id object in recentApplications) {
            SApplicationMenu *applicationMenu = [[SApplicationMenu alloc] initWithTitle:@"Menu"];
            NSMenuItem *menuItem = [_statusMenu insertItemWithTitle:object[@"name"] action:nil keyEquivalent:@"" atIndex:0];
            [menuItem setImage:[NSImage imageNamed:@"block"]];
            [menuItem setEnabled:YES];
            [menuItem setTag:55];
            [menuItem setSubmenu:applicationMenu];
            applicationMenu.object = object;
        }

    }
}

- (void)requestDidFinish:(SRequest *)request {

    if (request.tag == 1) {

        if ([request.json isKindOfClass:[NSDictionary class]]) {
            NSLog(@"application id : %@", (request.json)[@"application_id"]);
            NSLog(@"deployment id : %@", (request.json)[@"id"]);

            SDeployMonitor *monitor = [[SDeployMonitor alloc] initWithObject:request.json delegate:self];
            [_deploys addObject:monitor];
        }
        
    } else {

        if ([request.json isKindOfClass:[NSArray class]]) {
            NSMenuItem *menuItem = nil;

                NSMenu *allAppsMenu = [[NSMenu alloc] init];;

                for (NSDictionary *application in request.json) {
                    SApplicationMenu *applicationMenu = [[SApplicationMenu alloc] initWithTitle:@"Menu"];
                    menuItem = [allAppsMenu addItemWithTitle:application[@"name"] action:nil keyEquivalent:@""];
                    [menuItem setEnabled:YES];
                    [menuItem setImage:[NSImage imageNamed:@"block"]];
                    [menuItem setSubmenu:applicationMenu];
                    applicationMenu.object = application;                
                }

                menuItem = [_statusMenu itemWithTag:kTagRecentAppsMenu];
                [menuItem setSubmenu:allAppsMenu];
        }
    }
}

- (void)showNotification:(NSString *)title body:(NSString *)body {
    NSUserNotification *notification = [[NSUserNotification alloc] init];
    notification.title = title;
    notification.informativeText = body;
    notification.soundName = NSUserNotificationDefaultSoundName;
    [[NSUserNotificationCenter defaultUserNotificationCenter] deliverNotification:notification];
}

- (void)deployDidFinish:(SDeployMonitor *)deploy {
    [_deploys removeObject:deploy];
    NSLog(@"deployDidFinish : %d", deploy.result);
    NSLog(@"number of remaining deploys : %ld", (unsigned long)_deploys.count);

    NSString *notificationBody = @"All deployed.";
    if (_deploys.count != 0) {
        notificationBody = [NSString stringWithFormat:@"%ld deploy(s) are still running.", (unsigned long)_deploys.count];
    }
    
    [self showNotification:[NSString stringWithFormat:@"Deployment %@", deploy.statusString] body:notificationBody];
}

- (void)preferencesAction:(id)sender {
    [_tokenTextField setStringValue:token];

    [_window makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
}

- (void)saveAction:(id)sender {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:[_tokenTextField stringValue] forKey:@"token"];
    [defaults synchronize];
    token = [_tokenTextField stringValue];

    [_window orderOut:self];
}

- (void)closeAction:(id)sender {
    [_window orderOut:self];
}

- (void)quitAction:(id)sender {
    [NSApp terminate:self];
}

@end
