//
//  (C)2013 Mobiq, Inc.
//

#import "Global.h"
#import "AppDelegate.h"
#import "SApplicationMenu.h"
#import "SDeployMonitor.h"

@implementation SApplicationMenu

- (id)initWithTitle:(NSString *)aTitle {
    self = [super initWithTitle:aTitle];
    if (self) {
        self.delegate = self;

        _deployMenuItem = [self addItemWithTitle:NSLocalizedString(@"Deploy", nil) action:@selector(deployAction:) keyEquivalent:@""];
        _deployMenuItem.target = self;
        [_deployMenuItem setImage:[NSImage imageNamed:@"upload-cloud"]];

        [self addItem:[NSMenuItem separatorItem]];

        _progressIndicator = [[NSProgressIndicator alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 16.0f, 16.0f)];
        _progressIndicator.style = NSProgressIndicatorSpinningStyle;
        _progressIndicator.controlSize = NSSmallControlSize;

        NSMenuItem *menuItem = nil;
        menuItem = [self addItemWithTitle:@"Loading..." action:nil keyEquivalent:@""];
        menuItem.tag = 1;
        menuItem.view = _progressIndicator;

    }
    return self;
}

- (void)dealloc {
    self.delegate = nil;

    [_listRequest cancel];
    _listRequest = nil;

}

- (void)menuWillOpen:(NSMenu *)menu {

    _listRequest = [[SRequest alloc] initWithToken:token
                                        apiPath:[NSString stringWithFormat:@"applications/%@/deployments", _object[@"id"]]
                                       delegate:self];
    [_listRequest start];

    if (_progressIndicator) {
        _progressIndicator.frame = CGRectMake(0.0f, 0.0f, menu.size.width, 16.0f);

        [_progressIndicator performSelector:@selector(startAnimation:)
                                 withObject:self
                                 afterDelay:0.0
                                    inModes:@[NSEventTrackingRunLoopMode]];
    }

}

- (void)menuDidClose:(NSMenu *)menu {
    [_listRequest cancel];
    _listRequest = nil;
}

- (void)deployAction:(id)sender {

    BOOL exists = NO;
    for (id application in recentApplications) {
        if ([application[@"id"] isEqualToString:_object[@"id"]]) {
            exists = YES;
            break;
        }
    }

    if (!exists) [recentApplications addObject:_object];
    if ([recentApplications count] >= 5) [recentApplications removeLastObject];

    NSString *apiPath = [NSString stringWithFormat:@"applications/%@/deploy", _object[@"id"]];
    _deployRequest = [[SRequest alloc] initWithToken:token
                                                apiPath:apiPath
                                             delegate:[NSApplication sharedApplication].delegate];
    _deployRequest.data = [NSMutableDictionary dictionaryWithObject:@"deploy" forKey:@"command"];
    _deployRequest.tag = 1;
    [_deployRequest start];
}

- (void)menuAction:(NSMenuItem *)sender {
    id deployObject = [sender representedObject];

    NSString *url = [NSString stringWithFormat:@"https://manage.scalarium.com/applications/%@/deployments/%@",
                     _object[@"id"],
                     deployObject[@"id"]];
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:url]];
}

- (NSString *)timeString:(NSDate *)date {
    NSUInteger seconds = [[NSDate date] timeIntervalSinceDate:date];

    if (seconds >= 60*60*24) return [NSString stringWithFormat:@"%ld days ago", (unsigned long)(seconds / 60 / 60 / 24)];
    if (seconds >= 60*60) return [NSString stringWithFormat:@"%ld hours ago", (unsigned long)(seconds / 60 / 60)];
    if (seconds >= 60) return [NSString stringWithFormat:@"%ld mins ago", (unsigned long)(seconds / 60)];

    return [NSString stringWithFormat:@"%ld secs ago", (unsigned long)seconds];
}

- (void)requestDidFinish:(SRequest *)request {

    _progressIndicator = nil;
    while (true) {
        NSMenuItem *item = [self itemWithTag:1];
        if (!item) break;
        [self removeItem:item];
    }

    if ([request.json isKindOfClass:[NSArray class]]) {
        NSMenuItem *menuItem = nil;
        for (NSDictionary *deploy in request.json) {

            NSDate *date = [NSDate dateWithNaturalLanguageString:deploy[@"created_at"]];

            menuItem = [self addItemWithTitle:[self timeString:date] action:@selector(menuAction:) keyEquivalent:@""];
            menuItem.target = self;
            menuItem.tag = 1;
            if ([deploy[@"status"] isEqualToString:@"successful"]) menuItem.image = [NSImage imageNamed:@"tick-circle"];
            if ([deploy[@"status"] isEqualToString:@"running"]) menuItem.image = [NSImage imageNamed:@"clock"];
            if ([deploy[@"status"] isEqualToString:@"failed"]) menuItem.image = [NSImage imageNamed:@"exclamation-circle"];
            menuItem.representedObject = deploy;
            
        }

    }

    _listRequest = nil;
}

- (void)request:(SRequest *)request didFailWithError:(NSError *)error {
    _listRequest = nil;
}

@end
