//
//  (C)2013 Mobiq, Inc.
//

#import "SRequest.h"

static NSString *const kAPIEndPoint = @"https://manage.scalarium.com/api/%@";

@interface SRequest()
- (NSMutableURLRequest *)createRequest;
@end;

@implementation SRequest

- (id)init {
    self = [super init];
    if (self) {
        _buffer = [NSMutableData data];
    }
    return self;
}

- (id)initWithToken:(NSString *)token apiPath:(NSString *)apiPath delegate:(id)delegate {
    self = [self init];
    if (self) {
        self.token = token;
        self.apiPath = apiPath;
        self.delegate = delegate;
    }
    return self;
}

- (void)dealloc {
    NSLog(@"ScalariumRequest dealloc");

    [_connection cancel];
}

- (NSMutableURLRequest *)createRequest {
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
    [request setValue:_token forHTTPHeaderField:@"X-Scalarium-Token"];
    [request setValue:@"application/vnd.scalarium-v1+json" forHTTPHeaderField:@"Accept"];
    return request;
}

- (void)start {
    NSMutableURLRequest *request = [self createRequest];
    [request setURL:[NSURL URLWithString:[NSString stringWithFormat:kAPIEndPoint, _apiPath]]];

    if (_data) {
        [request setHTTPMethod:@"POST"];
        [request setHTTPBody:[NSJSONSerialization dataWithJSONObject:_data options:0 error:nil]];
    }

    _connection = [[NSURLConnection alloc] initWithRequest:request delegate:self startImmediately:NO];
    [_connection scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
    [_connection start];
}

- (void)cancel {
    [_connection cancel];
    _connection = nil;
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    [_buffer setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [_buffer appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    NSError *error = nil;
    _connection = nil;

    id json = [NSJSONSerialization JSONObjectWithData:_buffer options:0 error:&error];
    if (json) {
        self.json = json;
        if ([_delegate respondsToSelector:@selector(requestDidFinish:)]) {
            [_delegate requestDidFinish:self];
        }
    } else {
        if ([_delegate respondsToSelector:@selector(request:didFailWithError:)]) {
            [_delegate request:self didFailWithError:error];
        }
    }
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    _connection = nil;
    [_buffer setLength:0];
    if ([_delegate respondsToSelector:@selector(request:didFailWithError:)]) {
        [_delegate request:self didFailWithError:error];
    }
}

@end
